From 3e206b952b8011c4bab97c7c87ea693832137999 Mon Sep 17 00:00:00 2001
From: Heikki Linnakangas <heikki.linnakangas@iki.fi>
Date: Sun, 25 Feb 2024 13:33:07 +0200
Subject: [PATCH 1/1] Disable O_DIRECT on AIX

AIX apparently doesn't support the __attribute__((aligned(a)))
attribute, when a > 16. That means that PGIOAlignedBlock is not
correctly aligned. That showed up as an assertion failure when writing
pages with the new bulk_write.c facility, because it uses "static
const" of type PGIOAlignedBlock.

pg_prewarm.c also uses a static variable of type PGIOAlignedBlock, and
it's not clear why it hasn't tripped the assertion.

There surely would be ways to force the alignment on AIX, but at least
this should make the buildfarm green again, and it doesn't seem worth
spending more effort on this right now. We might soon drop AIX support
altogether unless someone steps up to the plate to maintain it
properly and set up a more recent buildfarm animal for it.

Discussion: https://www.postgresql.org/message-id/20240224195024.af@rfd.leadboat.com
---
 src/include/c.h          | 14 ++++++++++----
 src/include/port/aix.h   |  3 +++
 src/include/storage/fd.h |  2 +-
 3 files changed, 14 insertions(+), 5 deletions(-)

diff --git a/src/include/c.h b/src/include/c.h
index 2e3ea206e1..9a464bd592 100644
--- a/src/include/c.h
+++ b/src/include/c.h
@@ -186,6 +186,12 @@
 #define pg_attribute_noreturn() __attribute__((noreturn))
 #define pg_attribute_packed() __attribute__((packed))
 #define HAVE_PG_ATTRIBUTE_NORETURN 1
+
+/* Some platforms (AIX) support the aligned attribute only for small values */
+#if !defined(PG_MAX_ATTRIBUTE_ALIGN) || PG_MAX_ATTRIBUTE_ALIGN >= PG_IO_ALIGN_SIZE
+#define pg_attribute_io_aligned __attribute__((aligned(PG_IO_ALIGN_SIZE)))
+#endif
+
 #elif defined(_MSC_VER)
 /*
  * MSVC supports aligned.  noreturn is also possible but in MSVC it is
@@ -1123,8 +1129,8 @@ typedef union PGAlignedBlock
  */
 typedef union PGIOAlignedBlock
 {
-#ifdef pg_attribute_aligned
-	pg_attribute_aligned(PG_IO_ALIGN_SIZE)
+#ifdef pg_attribute_io_aligned
+	pg_attribute_io_aligned
 #endif
 	char		data[BLCKSZ];
 	double		force_align_d;
@@ -1134,8 +1140,8 @@ typedef union PGIOAlignedBlock
 /* Same, but for an XLOG_BLCKSZ-sized buffer */
 typedef union PGAlignedXLogBlock
 {
-#ifdef pg_attribute_aligned
-	pg_attribute_aligned(PG_IO_ALIGN_SIZE)
+#ifdef pg_attribute_io_aligned
+	pg_attribute_io_aligned
 #endif
 	char		data[XLOG_BLCKSZ];
 	double		force_align_d;
diff --git a/src/include/port/aix.h b/src/include/port/aix.h
index 5b1159c578..405151746d 100644
--- a/src/include/port/aix.h
+++ b/src/include/port/aix.h
@@ -12,3 +12,6 @@
 #if defined(__ILP32__) && defined(__IBMC__)
 #define PG_FORCE_DISABLE_INLINE
 #endif
+
+/* pg_attribute_aligned(n) works only up to 16 bytes on AIX. */
+#define PG_MAX_ATTRIBUTE_ALIGN	16
diff --git a/src/include/storage/fd.h b/src/include/storage/fd.h
index 60bba5c970..579cb3142b 100644
--- a/src/include/storage/fd.h
+++ b/src/include/storage/fd.h
@@ -88,7 +88,7 @@ extern PGDLLIMPORT int max_safe_fds;
  * idea on a Unix).  We can only use it if the compiler will correctly align
  * PGIOAlignedBlock for us, though.
  */
-#if defined(O_DIRECT) && defined(pg_attribute_aligned)
+#if defined(O_DIRECT) && defined(pg_attribute_io_aligned)
 #define		PG_O_DIRECT O_DIRECT
 #elif defined(F_NOCACHE)
 #define		PG_O_DIRECT 0x80000000
-- 
2.39.2

